Low-Diam-Routing Things
=======================

This repository contains

- Code to generate network topology graphs for Dragonfly, Fat tree, HyperX, Xpander and Jellyfish, as well as some ready-made Slim Fly instances (network-examples/, analysis-paths/).
- Code for theoretical analysis of topologies, mostly edge connectivity based (analysis-paths/).
- Code and configurations to simulate datacenter networks using OMNeT++/INET (requires patched INET and potentially OMNeT++ patches) (omnet/).
- A patched copy of htsim (with NDP) to run the same simulations on htsim (htsim/).
- Tools to analyze and plot results (analysis-paths/results.py).

Most of these can be built (or even run) by typing `make` in directories with `Makefiles`. More specific instructions follow below.

Preparing topologies
--------------------

Various scripts expect to find topology files to read and work on. The files use a simple text format:
```
<number of nodes> <number of edges>
<a> <b> <c> <... neighbors of node 1 (1-based)>
<a> <b> <c> <... neighbors of node 2 (1-based)>
...
```
To generate all required instances, do
```
cd analysis-paths/
python3 make_complete.py
python3 make_dragonfly.py
python3 make_fattree.py
python3 make_hyperx.py
python3 make_jellyfish.py
python3 make_xpander.py
```

Each will generate instances according to the configuration within each file. The naming convention is
```
<topology>.<radix>.adj.txt[.instance]
```
The `instance` field is used for randomized topologies, where multiple, randomized instances are generated per configuration. The `radix` field typically has the network radix or some related network parameter.

The Jellyfish instances use parameters and naming matching each of the other topologies.

Running the connectivity analysis
---------------------------------

Most scripts in `analysis-paths/` import their configuration from `countpaths.py`. This is primarily a list of topology files to work on. Results are saved using the `results` tool, the output file name and additional parameters are set in the `__main__` part of the script.

The most important one is currently `interference_deterministic.py`, which uses a deterministic, rather quick Ford-Fulkerson method for length-restricted edge connectivity. It is based on BFS implemented using SciPy sparse matrices (`scipy.sparse`). It computes not only interference, but also simple edge connectivity, which can be restricted to shortest paths using some SQL postprocessing, and is used for all connectivity results in the current reports.  

Running OMNeT++
---------------

First, install omnetpp-5.0. This should be sufficiently documented. Make sure to set the required environment variables. `omnet/sectionbasedconfig.[cc|h]` from this repo can be used to replace the OMNeT++ version, it completely removes vector recording to avoid a scalability problem there.

Next, install the modified INET from `git@spcl.inf.ethz.ch:herik/inet.git`. Checkout the `mptcp-3.4` branch (which does not actually contain working MPTCP at the moment). Use `./inet_featuretool` to disable most features (unless you enjoy installing lots of dependencies and compiling useless code):
```
grep 'true' .oppfeaturestate
    <feature enabled="true" id="TCP_common"/>
    <feature enabled="true" id="TCP_INET"/>
    <feature enabled="true" id="IPv4"/>
    <feature enabled="true" id="IPv6"/>
    <feature enabled="true" id="Generic"/>
    <feature enabled="true" id="Ethernet"/>
```

Build a release build (or debug, if you need to debug; but it will be slow):
```
 make MODE=release
```
If you ever get weird linker errors at build- or runtime, most likely you messed up debug- and release builds. `omnet/compile-omnet-inet.sh` contains additional hints on how to build for best performance (but it might not be worth it).

Now, to run INET, `cd omnet/` and run `make [-j <n>]`. But you should understand what that does:

The script `parameters/make_parameters.py` generates parameter files for all combinations of various parameter ranges. Edit the file to set up the parameters you want. Then generate the parameter files using `make parfiles`. The makefile will now build a `*.db` file from each `*.par` file in the `parameters/` folder by running `run_inet`, and finally merge the results into a single `results.db`. This means runs with many different parameter sets can be easily stopped and continued.

The `run_inet` script reads the parameter file and the global `omnetpp.ini`, and produces the specific configuration files for OMNeT and INET in a `work/*` folder. To do so, it will excecute various shell commands in `omnetpp.ini`, which invoke various `make_*.py` scripts to generate the more complicated parts. Then, it will invoke OMNeT (some configuration is passed from the Makefile in environment variables) to run the simulation, potentially even with MPI (if `NPROCS` is set). Finally, it will execute more commands from `omnetpp.ini`, which (partially) parse the OMNeT output files and funnel the results into the `results record` command. Note that you can get parallelism from `make`, or by using parsim and `NPROCS`, or even both. Usually using make is the better option, but for large instances you will run out of memory with many jobs; then use parsim.

For a full list of possible parameters, you will have to look into the various script files.

Running htsim
-------------

Our htsim has been forked into this repository immediatly after the NDP publication, the upstream is [here](https://github.com/nets-cs-pub-ro/NDP/wiki/NDP-Simulator).

Setting up htsim is *much* easier than setting up OMNeT++, but to work on it you will have to understand how discrete event simulation works; there is very little abstraction. It is not that complicated, though.

First:
```
cd htsim/
make   # this will fail to compile parse_output, for some reason; but we don't need it
cd datacenter/
make
cd ../batch
```

For tuning, set the `CXXFLAGS` in the Makefile. You should get some programs `htsim_*` in `datacenter/`.

Now, to run htsim, a similar setup as for OMNeT is used. This differs a bit from upstream htsim. In `batch/parameters/`, you find a `make_parameters.py` script that enumerates configurations, and writes parfiles. `make parfiles` in `batch/` runs it. `make` in `batch/` generates one `*.db` file from each parfile by running `run_htsim`, which takes the parfile, assembles a commandline for htsim, runs it, and records results using `results record`. Our htsim writes all results to stdout, so be prepared if you try to run it manually. `run_htsim` outputs all important commandlines, so you can run all steps manually for debugging.


