from countpaths import read_graph, graphnames
from results import Results
import numpy as np
import random

LIMIT=20

def count_collisions(pis, n, p, collect):
    acc = [0]*LIMIT
    for i in range(0, p*n, p):
        P = np.zeros([p*n])
        for pi in pis:
            P[pi[i:i+p]] += 1
        counts = P.reshape([n, p]).sum(1)
        for k in range(0, LIMIT):
            acc[k] += (counts == k).sum()
    for k in range(0, LIMIT):
        collect(multiplicity=k, count=acc[k])
        
def randperm(N):
    return [np.array(random.sample(range(0, N), N))]
        
def randadvk(N, k):
    pi = np.array(random.sample(range(0, N), N))
    invpi = np.zeros_like(pi)
    for i, j in enumerate(pi):
        invpi[j] = i
    advk = list(range(k, N)) + list(range(0, k))
    return [[pi[advk[invpi[i]]] for i in range(0, N)]]

def randstencil(N, k):
    pi = np.array(random.sample(range(0, N), N))
    invpi = np.zeros_like(pi)
    for i, j in enumerate(pi):
        invpi[j] = i
    plusk  = list(range(k, N)) + list(range(0, k))
    minusk = list(range(N-k, N)) + list(range(0, N-k))
    plus1  = list(range(1, N)) + list(range(0, 1))
    minus1 = list(range(N-1, N)) + list(range(0, N-1))
    return [[pi[advk[invpi[i]]] for i in range(0, N)] 
        for advk in [plusk, minusk, plus1, minus1]]

def randshuffle(N):
    pi = np.array(random.sample(range(0, N), N))
    invpi = np.zeros_like(pi)
    for i, j in enumerate(pi):
        invpi[j] = i
    pow2 = 2**int(np.ceil(np.log2(N)))
    shuf = np.array((((np.arange(N) * 2) % pow2) + np.floor(np.arange(N)/(pow2/2))) % N, dtype=np.int32)
    return [[pi[shuf[invpi[i]]] for i in range(0, N)]]

if __name__ == '__main__':
    r = Results('pathcollisions-shuffle.db')
    types = {" randperm":  lambda n: randperm(n),
             #" stride":    lambda n: randadvk(n, int(n**0.5)),
             " shuffle":    lambda n: randshuffle(n),
             #"randperm4": lambda n: randperm(n)+randperm(n)+randperm(n)+randperm(n),
             #"stencil":   lambda n: randstencil(n, int(n**0.5)),
             }
             
    for N_design in [1e3, 1e5]:
        for tag, func in types.items():
            for d in [1,2,3]:
                k = int((N_design*d)**(1.0/(d+1)))
                p = int(k/d)
                N_R = int(N_design / p)
                N = N_R*p
                print('tag =', tag, 'N =', N, 'p =', p, 'N_R =', N_R)
                collect = r.collector(multiplicity = Results.Int, count = Results.Int, n = int(N_R), concentration = int(p), diameter = int(d), tag=tag)
                for _ in range(50):
                    count_collisions(func(N), N_R, p, collect)
        r.commit()
    r.close()
                
            
