import numpy as np

edgetype = np.float64

# can replace arbitrary functions, esp. Result collector
def printit(*args, **kwargs):
    print(args, kwargs)

def read_graph(filename):
    f = open(filename)

    firstline = f.readline().split(" ")
    edges = int(firstline[1])
    vertices = int(firstline[0])
    
    graph = np.matrix(np.zeros((vertices, vertices)), dtype=edgetype)
    graph.vertices = vertices
    graph.radix = edges*2/vertices

    vertex = 1
    for line in f:
        s = line.split(" ")
        for i in range(len(s)-1):
            eid = int(s[i])
            graph[vertex-1, eid-1] += 1
        vertex=vertex+1
    if graph.sum() != edges*2:
        raise Exception("Malformed graph file!")
    
    if abs(round(graph.radix)-graph[0,:].sum()) < 2: # all fine, d-regular graph
        graph.edge = vertices
        graph.radix = round(graph.radix) # might be non-integer for JF
    else: # probably FatTree
        # radix is fractional but fine
        graph.edge = int(graph[0,:].sum())**2 * 2 # num edge routers
        
    return graph

dragonflies = [
            #"../network-examples/adj-format/DF.2.adj.txt",
            #"../network-examples/adj-format/DF.3.adj.txt",
            #"../network-examples/adj-format/DF.4.adj.txt",
            #"../network-examples/adj-format/DF.5.adj.txt",
            #"../network-examples/adj-format/DF.6.adj.txt",
            #"../network-examples/adj-format/DF.7.adj.txt",
            "../network-examples/adj-format/DF.8.adj.txt",
            #"../network-examples/adj-format/DF.12.adj.txt",
            ]
slimflies = [
            #"../network-examples/adj-format/MMS.5.adj.txt",
            #"../network-examples/adj-format/MMS.7.adj.txt",
            #"../network-examples/adj-format/MMS.11.adj.txt",
            #"../network-examples/adj-format/MMS.17.adj.txt",
            #"../network-examples/adj-format/MMS.19.adj.txt",
            #"../network-examples/adj-format/MMS.25.adj.txt",
            "../network-examples/adj-format/MMS.29.adj.txt",
            #"../network-examples/adj-format/MMS.35.adj.txt",
            #"../network-examples/adj-format/MMS.43.adj.txt",
            #"../network-examples/adj-format/MMS.47.adj.txt",
            #"../network-examples/adj-format/MMS.55.adj.txt",
            ]

xpanders = [
            "Xp.16.adj.txt.1",
            "Xp.32.adj.txt.1",
           ]
xpanders_p = [
            "Xpp.16.adj.txt.1",
            "Xpp.32.adj.txt.1",
            ]

hyperx_2 = [
            #"HyperX2.22.adj.txt",
            #"HyperX2.36.adj.txt",
            "HyperX2.42.adj.txt",
            #"HyperX2.52.adj.txt",
            #"HyperX2.62.adj.txt",
            ]
hyperx_3 = [
            #"HyperX3.15.adj.txt",
            "HyperX3.30.adj.txt",
            #"HyperX3.48.adj.txt",
            ]
hyperx_4 = [
            #"HyperX4.16.adj.txt",
            ]

fattree = [
            #"FatTree.4.adj.txt",
            #"FatTree.10.adj.txt",
            "FatTree.9.adj.txt",
            "FatTree.18.adj.txt",
            #"FatTree.36.adj.txt",
            ]

jellyfish_mms = [
            #"JF-MMS.5.adj.txt.1",
            #"JF-MMS.7.adj.txt.1",
            #"JF-MMS.11.adj.txt.1",
            #"JF-MMS.17.adj.txt.1",
            #"JF-MMS.19.adj.txt.1",
            #"JF-MMS.25.adj.txt.1",
            "JF-MMS.29.adj.txt.1",
            #"JF-MMS.35.adj.txt.1",
            #"JF-MMS.43.adj.txt.1",
            #"JF-MMS.47.adj.txt.1",
            #"JF-MMS.55.adj.txt.1",
            ]
            
jellyfish_hx2 = [
            #"JF-HX2.22.adj.txt.1",
            "JF-HX2.42.adj.txt.1",
            ]

jellyfish_df = [
            #"JF-DF.4.adj.txt.1",
            "JF-DF.8.adj.txt.1",
            #"JF-DF.12.adj.txt.1",
            ]
            
jellyfish_hx3 = [
            #"JF-HX3.15.adj.txt.1",
            "JF-HX3.30.adj.txt.1",
            #"JF-HX3.48.adj.txt.1",
            ]

jellyfish_ft = [
            #"JF-FT.4.adj.txt.1",
            #"JF-FT.10.adj.txt.1",
            "JF-FT.9.adj.txt.1",
            "JF-FT.18.adj.txt.1",
            #"JF-FT.36.adj.txt.1",
            ]
jellyfish_xp = [
            "JF-XP.16.adj.txt.1",
            "JF-XP.32.adj.txt.1",
]

complete = [
           #"Complete.10.adj.txt",
           "Complete.100.adj.txt",
           #"Bipartite.10.adj.txt",
           #"Bipartite.100.adj.txt",
           #"Tripartite.10.adj.txt",
           #"Tripartite.100.adj.txt"
           ]

graphnames = {
        'DF': (dragonflies, 1), 
        'MMS': (slimflies, 1), 
        'JF-MMS': (jellyfish_mms, 1), 
        'JF-DF': (jellyfish_df, 1), 
        'JF-HX2': (jellyfish_hx2, 1), 
        'JF-HX3': (jellyfish_hx3, 1), 
        #'JF-FT': (jellyfish_ft, 1), 
        #'JF-XP': (jellyfish_xp, 1), 
        #'XP': (xpanders, 1),
        #'XP-P': (xpanders_p, 1),
        'HX2': (hyperx_2,1),
        'HX3': (hyperx_3,1),
        #'HX4': (hyperx_4,1),
        #'FT': (fattree,1),
        'K': (complete, 1), 
    }

designd = {
        'DF': 3, 
        'MMS': 2, 
        'JF-MMS': 2, 
        'JF-HX2': 2, 
        'JF-DF': 3, 
        'JF-HX3': 3, 
        'JF-FT': 4, 
        'JF-XP': 2, 
        'XP': 2,
        'XP-P': 2,
        'HX2': 2,
        'HX3': 3,
        'HX4': 4,
        'FT': 4,
        'K': 1, 
}
    
