from countpaths import *
import numpy as np
from scipy.sparse import *
import random
from simplepmap import pmap
from results import Results

# parallel matrix multiplication
def pmult(A, B):
    A = A.asformat("csr")
    BS = 1000 
    nblocks = int(A.shape[0] / BS)
    aa = [A[i*BS:(i+1)*BS, :] for i in range(0, nblocks)]
    if nblocks*BS < A.shape[0]:
        aa.append(A[nblocks*BS:, :])
    qs = pmap(lambda a: a*B, aa)
    return vstack(qs)

def countpaths(g, limit, collect, count = 50000):

    if g.edge**2 > count:
        r = np.arange(0, g.edge, 1)
        pairs = [random.sample(list(r), 2) for _ in range(0, count)]
    else:
        pairs = [(i, j) for i in range(g.edge) for j in range(g.edge) if i != j]
        
    def edge(i, j):
        return i*g.vertices + j
    
    blocks = [g[:,k].dot(g[k,:]) for k in range(g.vertices)]
    k = block_diag(blocks, format='csc')
    permuted = [k.getcol(edge(j, i)) for i in range(g.vertices) for j in range(g.vertices)]
    k = hstack(permuted)
    nonzero = k.nonzero()
    k[nonzero] = np.random.sample(len(nonzero[0]))
    
    K = identity(g.vertices**2).asformat("csr")
    
    for i in range(1, limit+1):
        print("rank")
        def rank(st):
            s,t = st
            def edge(i, j):
                return i*g.vertices + j
            cols = [edge(s,k) for k,e in enumerate(g[s,:].flat) if e > 0]
            rows = [edge(k,t) for k,e in enumerate(g[t,:].flat) if e > 0]
            # indexing via the meshgrid is faster than slicing the sparse matrix
            x,y  = np.meshgrid(rows, cols)
            sub  = K[x.flatten(), y.flatten()].reshape(x.shape)
            return np.linalg.matrix_rank(sub)
        out = pmap(rank, pairs)

        print("collect")
        for c, (s, t) in zip(out, pairs):
            collect(len=i, count=c, s=s, t=t)

        if i == limit: break
        print("multiply")
        K = pmult(K, k) + identity(g.vertices**2)
        
if __name__ == '__main__':
    r = Results('xpander.db')
    for topo, (files, samp) in graphnames.items():
        for i in range(1,samp+1):
            for name in files:
                if i != 1:
                    n = name[:-1] + str(i)
                else:
                    n = name
                print(n)
                g = read_graph(n)
                g.topo = topo
                g.sample = i
                p = round(g.radix / designd[topo] * g.vertices / g.edge)

                collect = r.collector(s=Results.Int, t=Results.Int, len=Results.Int, count=Results.Int, topo=g.topo, n_r=g.vertices, radix=g.radix, n_e=g.edge*p, p=p, designd=designd[topo], sample=i)
                countpaths(g, 5, collect)
                r.commit()
    r.close()

