from countpaths import read_graph, graphnames
import numpy as np
from scipy.sparse import *
import random
from simplepmap import pmap
from results import Results

# parallel matrix multiplication
def pmult(A, B):
    A = A.asformat("csr")
    BS = 10000 
    nblocks = int(A.shape[0] / BS)
    aa = [A[i*BS:(i+1)*BS, :] for i in range(0, nblocks)]
    if nblocks*BS < A.shape[0]:
        aa.append(A[nblocks*BS:, :])
    qs = pmap(lambda a: a*B, aa)
    return vstack(qs)

def countpaths(g, limit, collect, count = 10000):

    r = np.arange(0, g.vertices, 1)
    pairs = [random.sample(list(r), 4) for _ in range(0, count)]
        
    def edge(i, j):
        return i*g.vertices + j
    
    blocks = [g[:,k].dot(g[k,:]) for k in range(g.vertices)]
    k = block_diag(blocks, format='csc')
    permuted = [k.getcol(edge(j, i)) for i in range(g.vertices) for j in range(g.vertices)]
    k = hstack(permuted)
    nonzero = k.nonzero()
    k[nonzero] = np.random.sample(len(nonzero[0]))
    
    K = identity(g.vertices**2).asformat("csr")
    
    for i in range(1, limit+1):
        print("rank")
        def rank(st):
            ss = st[ :len(st)//2]
            tt = st[len(st)//2: ]
            
            def edge(i, j):
                return i*g.vertices + j
            cols = [edge(s,k) for s in ss for k,e in enumerate(g[s,:].flat) if e > 0]
            rows = [edge(k,t) for t in tt for k,e in enumerate(g[t,:].flat) if e > 0]
            x,y  = np.meshgrid(rows, cols)
            sub  = K[x.flatten(), y.flatten()].reshape(x.shape)
            total = np.linalg.matrix_rank(sub)
            
            individual = 0
            for s in ss:
                for t in tt:
                    cols = [edge(s,k) for k,e in enumerate(g[s,:].flat) if e > 0]
                    rows = [edge(k,t) for k,e in enumerate(g[t,:].flat) if e > 0]
                    x,y  = np.meshgrid(rows, cols)
                    sub  = K[x.flatten(), y.flatten()].reshape(x.shape)
                    individual += np.linalg.matrix_rank(sub)
                
            return (individual, total)
        
        out = pmap(rank, pairs)

        print("collect")
        for (c_individual, c_total) in out:
            collect(len=i, count_individual=c_individual, count_total=c_total)

        if i == limit: break
        print("multiply")
        K = pmult(K, k) + identity(g.vertices**2)
        
if __name__ == '__main__':
    r = Results('sets.db')
    for topo, (files, samp) in graphnames.items():
        for i in range(1,samp+1):
            for name in files:
                if i != 1:
                    n = name[:-1] + str(i)
                else:
                    n = name
                print(n)
                g = read_graph(n)
                
                collect = r.collector(len=Results.Int, count_individual=Results.Int, count_total=Results.Int, topo=topo, n=g.vertices, radix=g.radix, sample=i)
                countpaths(g, 5, collect)
                r.commit()
    r.close()

#   from ggplot import *
#   (ggplot(data, aes(x='count_total', fill='factor(len)')) 
#       + geom_histogram(bins=np.arange(-0.5, 60.5, 1), alpha=0.5)
#       + facet_grid('topo', 'len')
#       #+ ggtitle("Total number of disjoint paths in random $K_{2,2}$ subgraphs")
#       + scale_y_log(base=10)
#       + ylim(low=1, high=1e4)
#   ).save("edge_disjoint_sets_total.pdf", width=20, height=24)
#   data['path_loss'] = data['count_individual'] - data['count_total']
#   (ggplot(data, aes(x='path_loss', fill='factor(len)')) 
#       + geom_histogram(bins=np.arange(-0.5, 60.5, 1), alpha=0.5)
#       + facet_grid('topo', 'len')
#       #+ ggtitle("Number of disjoint paths lost in random $K_{2,2}$ subgraphs relative to individual edges")
#       + scale_y_log(base=10)
#       + ylim(low=1, high=1e4)
#   ).save("edge_disjoint_sets_loss.pdf", width=20, height=24)

