from countpaths import read_graph, graphnames
import numpy as np
from scipy.sparse import *
import random
from simplepmap import pmap
from results import Results

# parallel matrix multiplication
def pmult(A, B):
    A = A.asformat("csr")
    BS = 1000 
    nblocks = int(A.shape[0] / BS)
    aa = [A[i*BS:(i+1)*BS, :] for i in range(0, nblocks)]
    if nblocks*BS < A.shape[0]:
        aa.append(A[nblocks*BS:, :])
    qs = pmap(lambda a: a*B, aa)
    return vstack(qs)

def growset(g, length, size, collect):

    def edge(i, j):
        return i*g.vertices + j
    
    blocks = [g[:,k].dot(g[k,:]) for k in range(g.vertices)]
    k = block_diag(blocks, format='csc')
    permuted = [k.getcol(edge(j, i)) for i in range(g.vertices) for j in range(g.vertices)]
    k = hstack(permuted)
    nonzero = k.nonzero()
    k[nonzero] = np.random.sample(len(nonzero[0]))
    
    K = (k + identity(g.vertices**2)).asformat("csr")
    
    assert length >= 1
    for i in range(1, length):
        print("multiply")
        K = pmult(K, k) + identity(g.vertices**2)
        
    def rank(subset):
        edges = [edge(s,t) for s in subset for t,e in enumerate(g[s,:].flat) if e > 0]
        nodes = [s         for s in subset for t,e in enumerate(g[s,:].flat) if e > 0]

        x,y = np.meshgrid(edges, edges)
        sub  = K[x.flatten(), y.flatten()].reshape(x.shape)
        x,y = np.meshgrid(nodes, nodes)
        sub[x==y] = 0

        total = np.linalg.matrix_rank(sub)
        return total, subset
    
    currentsubset = [random.randint(0, g.vertices-1)]
    
    for i in range(1, size):
        tests = [currentsubset + [k] for k in range(0, g.vertices) if k not in currentsubset]
        print("Size %d..." % i)
        res = pmap(rank, tests)
        res.sort()
        for count, subset in res:
            collect(len=length, count=count, size=len(subset), subset=" %s " % " ".join(map(str,subset)))
        print("opt: %d, %s" % (res[0][0], res[0][1]))
        currentsubset = res[0][1]
            
if __name__ == '__main__':
    r = Results('sets.db')
    for topo, (files, samp) in graphnames.items():
        for i in range(1,samp+1):
            for name in files:
                if i != 1:
                    n = name[:-1] + str(i)
                else:
                    n = name
                print(n)
                g = read_graph(n)
                
                collect = r.collector(len=Results.Int, count=Results.Int, size=Results.Int, subset=Results.Text, topo=topo, n=g.vertices, radix=g.radix, sample=i)
                growset(g, 2, 10, collect)
                growset(g, 3, 10, collect)
                growset(g, 4, 10, collect)
                r.commit()
    r.close()
