from countpaths import *
import numpy as np
from scipy.sparse import *
import random
from simplepmap import pmap
from results import Results

# parallel matrix multiplication
def pmult(A, B):
    A = A.asformat("csr")
    BS = 1000 
    nblocks = int(A.shape[0] / BS)
    aa = [A[i*BS:(i+1)*BS, :] for i in range(0, nblocks)]
    if nblocks*BS < A.shape[0]:
        aa.append(A[nblocks*BS:, :])
    qs = pmap(lambda a: a*B, aa)
    return vstack(qs)

def countpaths(g, limit, collect, count = 5000):

    r = list(range(0, g.edge))
    pairs = [random.sample(r, 4) for _ in range(0, count)]
        
    def edge(i, j):
        if i > j: i, j = j, i
        return i*g.shape[0] + j
    
    k = dok_matrix((g.shape[0]**2, g.shape[0]**2))
    for s in range(g.shape[0]):
        out = g[:,s].nonzero()[0]
        for t1 in out:
            for t2 in out:
                k[edge(s,t1), edge(s,t2)] = 1
    nonzero = k.nonzero()
    k[nonzero] = 0.9 + np.random.sample(len(nonzero[0]))*0.2
    k = k.asformat("csc")
    
    K = identity(g.vertices**2).asformat("csr")
    
    for i in range(1, limit+1):
        print("rank")
        def rank(X, Y):
            cols = sum(([edge(s,k) for k,e in enumerate(g[s,:].flat) if e > 0] for s in X), [])
            rows = sum(([edge(k,t) for k,e in enumerate(g[t,:].flat) if e > 0] for t in Y), [])
            # indexing via the meshgrid is faster than slicing the sparse matrix
            x,y  = np.meshgrid(rows, cols)
            sub  = K[x.flatten(), y.flatten()].reshape(x.shape)
            return np.linalg.matrix_rank(sub)

        def xabcd(abcd):
            a,b,c,d = abcd
            c_acb = rank([a,c], [b])
            c_acd = rank([a,c], [d])
            c_abcd = rank([a,c], [b,d])
            return c_acb+c_acd-c_abcd, c_abcd, c_acd, c_acb

        out = pmap(xabcd, pairs)

        print("collect")
        for (x_abcd, c_abcd, c_acd, c_acb), (a,b,c,d) in zip(out, pairs):
            collect(len=i, a=a, b=b, c=c, d=d, x_abcd=x_abcd, c_abcd=c_abcd, c_acd=c_acd, c_acb=c_acb)

        if i == limit: break
        print("multiply")
        K = pmult(K, k) + identity(g.vertices**2)
        
if __name__ == '__main__':
    r = Results('interference-validation.db')
    for topo, (files, samp) in graphnames.items():
        for i in range(1,samp+1):
            for name in files:
                if i != 1:
                    n = name[:-1] + str(i)
                else:
                    n = name
                print(n)
                g = read_graph(n)
                g.topo = topo
                g.sample = i
                p = round(g.radix / designd[topo] * g.vertices / g.edge)

                collect = r.collector(a=Results.Int, b=Results.Int, c=Results.Int, d=Results.Int, 
                                      x_abcd=Results.Int, c_abcd=Results.Int, c_acd=Results.Int, c_acb=Results.Int,
                                      len=Results.Int, topo=g.topo, n_r=g.vertices, radix=g.radix, n_e=g.edge*p, p=p, designd=designd[topo], sample=i)
                countpaths(g, 5, collect)
                r.commit()
    r.close()

