from countpaths import *
import numpy as np
from scipy.sparse import *
import random
from simplepmap import pmap
from results import Results

bits = np.array(2**np.arange(50), dtype=np.int64)

def prepare_graph(g):
    h = csr_matrix(g)
    for i in range(g.shape[0]):
        idx = h[i,:].nonzero()[1]
        h[i,idx] = 2**np.arange(len(idx))
    return h

def bfs(X,Y, limit, h):
    fringe = np.zeros((h.shape[0],))
    seen = np.zeros((h.shape[0],))
    fringe[X] = 1
    seen[X] = -1
    for i in range(limit):
        step = h.dot(fringe)
        new = (step > 0) & (seen == 0)
        fringe[new] = 1
        seen[new] = step[new]
        if new[Y].any():
            break
    found = seen[Y] > 0
    if not found.any():
        return None
    cur = Y[found.nonzero()[0][0]]
    path = []
    while cur not in X:
        nextport = ((bits & np.int64(seen[cur])) > 0).nonzero()[0][0]
        next = h[cur,:].indices[nextport]
        path.append((cur, next))
        cur = next
    return path

def c_XY(X,Y, limit, h):
    g = h.tocsr(copy=True)
    res = np.zeros((limit+1,), dtype=np.int32)
    while True:
        p = bfs(X,Y, limit, g)
        if not p: break
        res[len(p):] += 1
        for s,t in p:
            g[s,t] = 0
            g[t,s] = 0
    return res

def countpaths(g, limit, collectc, collectx, count = 100):

    r = list(range(0, g.edge))
    pairs = [random.sample(r, 4) for _ in range(0, count)]

    h = prepare_graph(g)

    def doall(abcd):
        a,b,c,d = abcd
        c_ab = c_XY([a], [b], limit, h)
        c_cd = c_XY([c], [d], limit, h)
        c_acb = c_XY([a,c], [b], limit, h)
        c_acd = c_XY([a,c], [d], limit, h)
        c_acbd = c_XY([a,c], [b,d], limit, h)
        return c_ab, c_cd, c_acb, c_acd, c_acbd

    out = pmap(doall, pairs)
        
    for (a,b,c,d), (c_ab, c_cd, c_acb, c_acd, c_acbd) in zip(pairs, out):
        for i in range(1, limit+1):
            collectc(len=i, a=a, b=b, c_ab=c_ab[i])
            collectc(len=i, a=c, b=d, c_ab=c_cd[i])
            collectx(len=i, a=a, b=b, c=c, d=d, x_abcd=c_acd[i]+c_acb[i]-c_acbd[i], c_abcd=c_acbd[i], c_acd=c_acd[i], c_acb=c_acb[i])
        
if __name__ == '__main__':
    r = Results('interference-deterministic.db')
    for topo, (files, samp) in graphnames.items():
        for i in range(1,samp+1):
            for name in files:
                if i != 1:
                    n = name[:-1] + str(i)
                else:
                    n = name
                print(n)
                g = read_graph(n)
                g.topo = topo
                g.sample = i
                p = round(g.radix / designd[topo] * g.vertices / g.edge)

                collectx = r.collector(a=Results.Int, b=Results.Int, c=Results.Int, d=Results.Int, 
                    x_abcd=Results.Int, c_abcd=Results.Int, c_acd=Results.Int, c_acb=Results.Int,
                    len=Results.Int, 
                    topo=g.topo, n_r=g.vertices, radix=g.radix, n_e=g.edge*p, p=p, designd=designd[topo], sample=i, tag="interference")
                collectc = r.collector(a=Results.Int, b=Results.Int,
                    c_ab=Results.Int, len=Results.Int, 
                    topo=g.topo, n_r=g.vertices, radix=g.radix, n_e=g.edge*p, p=p, designd=designd[topo], sample=i, tag="connectivity")
                countpaths(g, 5, collectc, collectx)
                r.commit()
    r.close()

