def make_fattree(p, d):
    assert d == 4 # only 3-level trees for the moment
    k = p*2
    n = 5*p**2
    N = 2*p**3
    
    graph = [[] for _ in range(n)]
    
    # the FT is a mirror-symmetric structre of 5 groups of p*p nodes
    node = enumerate(graph)
    def p_square_group():
        return [[next(node) for _ in range(p)] for __ in range(p)]
    
    edge_left  = p_square_group() # the edge has the low node numbers
    edge_right = p_square_group()
    aggr_left  = p_square_group()
    aggr_right = p_square_group()
    core = p_square_group()
    
    for ee, aa in [(edge_left, aggr_left), (edge_right, aggr_right)]:
        # edge and aggregation are fully connected bipartite
        for e, a in zip(ee, aa):
            for i in range(p):
                for j in range(p):
                    a[i][1].append(e[j][0])
                    e[j][1].append(a[i][0])
        # each aggregation group connects to each core group, with one link per member
        for c in core:
            for a in aa:
                for i in range(p):
                    c[i][1].append(a[i][0])
                    a[i][1].append(c[i][0])
    return graph
    
def output_graph(graph, file=None):
    print(len(graph), int(sum(len(n) for n in graph) / 2), file=file)
    for node in graph:
       print( " ".join(str(e+1) for e in node) + " ", file=file)
        
if __name__ == '__main__':
    networks = [(9, 4, "FatTree.9.adj.txt"), 
                (18, 4, "FatTree.18.adj.txt"), 
                (36, 4, "FatTree.36.adj.txt"), 
                (8, 4, "FatTree.8.adj.txt"), 
                (16, 4, "FatTree.16.adj.txt"), 
                (26, 4, "FatTree.26.adj.txt"), 

                (2, 4, "FatTree.2.adj.txt"), 
               ]
    for n in networks:
        print(n)
        with open(n[2], "w") as f:
            g = make_fattree(n[0], n[1])
            output_graph(g, file=f)
    
