import random

# simple integer factorization -- because we can
def onefactor(x, q):
    if x % q == 0: return (int(x/q), q)
    return onefactor(x, q+1)

def factors(x, q = 2):
    if x == 1: return []
    else:
        y, p = onefactor(x, q)
        return [p] + factors(y, p)

def inv(perm):
    inverse = [0] * len(perm)
    for i, p in enumerate(perm):
        inverse[p] = i
    return inverse

# fully connected graphs
def K(n):
    return [[j for j in range(n) if j != i] for i in range(n)]

# random k-lift of g
def lift(g, k):
    perms = {}
    def perm(e1, e2):
        if (e1, e2) not in perms:
            p = list(range(k))
            random.shuffle(p)
            perms[(e1, e2)] = p
            perms[(e2, e1)] = inv(p)
        return perms[(e1, e2)]
    
    # copy g k-times (loop order is strange here), rewire edges by random matching
    G = [[e2 + perm(e1, e2)[i]*len(g) for e2 in node] for i in range(k) for e1, node in enumerate(g)]
    return G

# constructs an xpander of approximate size; Precise if n%(r+1)=0.
def make_xpander(r, fact):
    g = K(r+1)
    print("factors: %s" % fact)
    for q in fact:
        g = lift(g, q)
    return g
        
def output_graph(graph, file=None):
    print(len(graph), int(sum(len(n) for n in graph) / 2), file=file)
    for node in graph:
       print( " ".join(str(e+1) for e in node) + " ", file=file)
        
if __name__ == '__main__':
    networks = [(16, factors(8), "Xp.16.adj.txt"),
                (16,        [8], "Xpp.16.adj.txt"),
                (32, factors(32), "Xp.32.adj.txt"),
                (32,        [32], "Xpp.32.adj.txt"),

                (4,        [5], "Xp.5.adj.txt"),
    ]

    nsamp = 3
    random.seed(42)
    
    for n in networks:
        print(n)
        for i in range(1, nsamp+1):
            with open("%s.%i" % (n[2], i), "w") as f:
                g = make_xpander(n[0], n[1])
                output_graph(g, file=f)
    
