from collections import defaultdict
from igraph import Graph
from itertools import takewhile, count
from pickle import dump
from string import split
from sys import exit, argv

def read_graph(filename):
    f = open(filename)
    graph = Graph()

    firstline = split(f.readline(), " ")
    edges = int(firstline[1])
    vertices = int(firstline[0])

    graph.add_vertices(vertices)

    vertex = 1
    for line in f:
        edges = list()
        s = split(line, " ")
        for i in range(0, len(s)-1):
            eid = int(s[i])
            if eid > vertex:
                edges.append((vertex-1, int(s[i])-1))
        vertex=vertex+1
        graph.add_edges(edges)
    return graph

def all_pairs_shortest_paths(graph):
    all_paths = dict()
    def group(paths):
        r = {}
        for p in paths:
            if not p[-1] in r:
                r[p[-1]] = []
            r[p[-1]].append(p)
        return r
    for s in graph.vs:
        all_paths[s.index] = group(graph.get_all_shortest_paths(s))
    return all_paths

def dot(X, Y):
  return [[sum(a*b for a,b in zip(X_row,Y_col)) for Y_col in zip(*Y)] for X_row in X]

def path_lengths():
    path_len_1 = graph.get_adjacency()
    path_len_2 = dot(path_len_1,path_len_1)
    path_len_3 = dot(path_len_2,path_len_1)

    for i in range(0, graph.vcount()):
      print ">>>>>>> Analyze paths from router ", i
      for j in range(0, graph.vcount()):
        if i == j:
          continue
        print ">>> Paths to router ", j, ":"
        print "Of length 1: ", path_len_1[i][j]
        print "Of length 2: ", path_len_2[i][j]
        print "Of length 3: ", path_len_3[i][j]


def paths_distinct(p1, p2):
    p1_e = set()
    p2_e = set()
    l = p1[0]
    for v in p1[1:]:
        p1_e.add((l, v))
        l = v
    l = p2[0]
    for v in p2[1:]:
        p2_e.add((l, v))
        l = v
    for e in p1_e:
        if e in p2_e:
            return False
    for e in p2_e:
        if e in p1_e:
            return False
    return True

def distinct(paths):
    result = set()
    for p in sorted(paths,key=len):
        d = True
        for s in result:
            if p == s:
                continue
            if not paths_distinct(p,s):
                d = False
                break
        if d:
            result.add(p)
    return result


def divide(ps):
    paths = dict()
    for i in range(1,200):
        paths[i] = set()
    for p in ps:
        if len(p) >= 2:
            paths[len(p)-1].add(p)
    return paths

def print_nice(s, ps):
    paths = divide(ps)
    print "%s  %2d, %2d, %2d, %3d" % (s, len(paths[1]),len(paths[2]), len(paths[3]), len(paths[4]))

# Returns a list of all loop-free paths between s and d
def find_all_paths(s, d, graph, max_len=4):
    result = set()
    stack = list()
    stack.append({'node': s, 'dist': 0, 'path': (s,)})
    while len(stack):
        s = stack.pop()
        if s['node'] == d and s['dist'] <= max_len:
            result.add(s['path'])
        if s['dist'] >= max_len:
            continue
        for n in graph.neighbors(s['node']):
            # No loops
            if not n in s['path']:
                stack.append({'node': n, 'dist': s['dist'] + 1, 'path': s['path'] + (n,)})
    return result

def eset_to_vset(eset):
    l = None
    result = list()
    for e in eset:
        result.append(e[0])
        l = e[1]
    result.append(l)
    return tuple(result)


def find_disjoint_paths(src, dst, graph):
    mf = graph.maxflow(src, dst)
    g = Graph(directed=True)
    g.add_vertices(graph.vcount())
    for i, v in enumerate(mf.flow):
        e = graph.es[i]
        if v > 0.5:
            g.add_edge(e.source, e.target)
        elif v < -0.5:
            g.add_edge(e.target, e.source)
    print "Edges in disjoint paths: %d" % g.ecount()

    result = list()
    for i in range(0, int(mf.value)):
        path = g.get_shortest_paths(src, dst, output="epath")[0]
        p = tuple([(g.es[x].source, g.es[x].target) for x in path])
        result.append(eset_to_vset(p))
        g.delete_edges(path)
    return result

def paths_len_dict(paths):
    result = dict()
    for p in paths:
        l = len(p) - 1
        if l in result:
            result[l] += 1
        else:
            result[l] = 1
    return result


def func_distinct_paths():
    results = list()
    #print "(Src, Dst) "
    m = {'l':0}
    for s in graph.vs:
        for d in graph.vs:
            if s == d:
                continue
            if src_index != dst_index and (s.index != src_index or d.index != dst_index):
                continue
            paths = []
            dpaths = []
            #paths = find_all_paths(s.index, d.index, graph)
            minpaths = graph.get_all_shortest_paths(s, d)
            if len(minpaths) > m['l']:
                m = {'l': len(minpaths), 's':s.index, 'd':d.index, 'n': minpaths}
            #print_nice("(%3d,%3d) All paths:          " % (s.index, d.index), paths)
            #dpaths = find_disjoint_paths(s.index, d.index, graph)
            #print_nice("          Disjoint paths (%2d):" % len(dpaths), dpaths)
            results.append(dict({'pair': "%d-%d" % (s.index, d.index)}.items() + \
                    {'path_len_%d' % k : v for k,v in paths_len_dict(paths).items()}.items() + \
                    {'disjoint_path_len_%d' % k : v for k,v in paths_len_dict(dpaths).items()}.items()))
        #break
    print m

    output_file = split(argv[1],'/')[-1] + "-results.py"
    f = open(output_file, 'w')
    dump(results, f)

def n_minimal_paths(graph, minpaths=None):
    if not minpaths:
        minpaths = all_pairs_shortest_paths(graph)
    results = list()
    #print "(Src, Dst) "
    m = {'l':0}
    n = {}
    for s in graph.vs:
        for d in graph.vs:
            if s == d:
                continue
            ms = minpaths[s.index][d.index]
            if not len(ms) in n:
                n[len(ms)] = 0
            n[len(ms)] += 1;
            if len(ms) > m['l']:
                m = {'l': len(ms), 's':s.index, 'd':d.index, 'n': ms}
    print m
    #print n
    s = sum(n.values()) * 1.0
    r = 0
    for i,v in n.iteritems():
        if i > 2:
            r += v
    return {'1': n[1]/s, '2': n[2]/s if 2 in n else 0.0, '>2': r/s}

def print_n_minimal_paths(result):
    print "1: %.2f, 2:%.2f, >2: %.2f" % (result['1'], result['2'], result['>2'])



def test():
    find_disjoint_paths(3,17, graph)
    for s in graph.vs:
        for d in graph.vs:
            if s == d:
                continue
            #find_disjoint_paths(s.index,d.index, graph)

def main():
    if len(argv) < 2:
      exit("usage: "+argv[0]+" <network structure in the adjacency list format>");

# First, we load the network graph from file

    f = open(argv[1])
    graph = Graph()

    firstline = split(f.readline(), " ")
    edges = int(firstline[1])
    vertices = int(firstline[0])

    graph.add_vertices(vertices)

    vertex = 1
# read input graph
    for line in f:
        edges = list()
        s = split(line, " ")
        for i in range(0, len(s)-1):
            eid = int(s[i])
            if eid > vertex:
                edges.append((vertex-1, int(s[i])-1))
        vertex=vertex+1
        graph.add_edges(edges)
        #print "====", len(s)

    print graph
    all_paths = all_pairs_shortest_paths(graph)

    src_index, dst_index = 0, 0
    if len(argv) == 4:
        src_index, dst_index = int(argv[2]), int(argv[3])

    #func_distinct_paths()
    #test()
    n_minimal_paths(graph)

if __name__ == '__main__':
    main()
