# Collection of plot snippets that might be useful.
alias result='python3 results.py'

# big map of histograms
results ggplot2 -r 'file="vertex_disjoint_paths.py"' -w "(topo != 'JF' or n=722)" -s "count as [X HIST INT LOGY; Disjoint path count], topo || ' n=' || n || ' k=' || radix as [YFACET; Network], len as [COLOR XFACET; len]" --size 15x15

# Overlaid histogram of the interesting cases (matplotlib histogram)
results ggplot -r 'file="edge_disjoint_paths.py"' -w "(len=3 and radix=29 or len=4 and radix=14) and (topo != 'JF' or n=722)" -s "count as [X HIST INT LINE; Disjoint path count], topo || ' n=' || n || ' k=' || radix as [COLOR; Network]" --size 5x5

# Same but drawn as a scatterplot. Nicer due to the markers, but legend is broken.
# the 20000 samples are hardcoded, one could use a subquery but...
results ggplot2 -r 'file="vertex_disjoint_paths.py"' -w "(len=3 and radix=29 or len=4 and radix=14) and (topo != 'JF' or n=722)" -s "count as [X BAR; Disjoint path count], count(*)/2e4 as [Y LOG; pairs (%)], topo || ' n=' || n || ' k=' || radix as [SHAPE COLOR; Network]" -g -i s -i t --size 5x5

# shortest path multiplicity plot from the very beginning
# This loses the errorbars.
results ggplot -r 'tag="shortest-path"' --datasql 'SELECT avg(summed*1.0/(n*(n-1))) as [Y; pairs(%)], n as [X LINE POINT; n], topo as [COLOR; Topology], multiplicity as [SHAPE; Multiplicity] FROM (SELECT sum(pairs) AS summed, n, topo, multiplicity FROM data GROUP BY n, topo, multiplicity, sample) GROUP BY n, topo, multiplicity' --size 6x5

# same using scatter and facet -- facet looks more tidy, actually this should be a profile.
results ggplot -r 'tag="shortest-path"' -s 'n as [X POINT LINE; n], sum(pairs)*1.0/(n*(n-1)) as [Y; pairs(%)], topo as [FACET; topo], multiplicity as [SHAPE; multi], len' -g -i len -i pairs

# tail disjoint paths. Works with min and avg. Samples not necessarily handled correctly.
results ggplot -r 'tag="tail-disjoint-count" and n > 500' -s 'topo as [COLOR; topo], n as [SHAPE; n], len as [X LINE POINT; len], avg(count)*1.0/radix as [Y; ports (%)]' -g -i count -i sample

# standard edge density plot
results ggplot -r 'tag="shortest-path" and topo not like "JF%"' --datasql "SELECT DISTINCT topo as [COLOR; topo], n as [X LINE POINT; n], n*1.0/radix/radix as [Y; $\frac{n}{k'^2}$] FROM current" --size 5x5

# average shortest path len
results ggplot -r 'tag="shortest-path"' -s 'topo as [COLOR SHAPE; topo], sum(len*pairs)*1.0/sum(pairs) as [Y; mean diameter], n as [X LINE POINT; n]' -g -i multiplicity -i len -i pairs -i sample --size 5x5

# histogram of shortest path lengths.
results ggplot2 -r 'tag="shortest-path" and n > 1000 and n < 2000' -s 'len as [X; len], sum(pairs)*1.0/(n*(n-1)) as [Y POINT BAR LINE; pairs(%)], topo || " n=" || n as [SHAPE COLOR; topo]' -g -i multiplicity  -i pairs --size=5x5

# edge-disjoint sets with loss and total in one plot
results ggplot2 -r 'file="edge_disjoint_sets.py"' --datasql "SELECT count_total as [X; Disjoint path count], count(*) as [Y LINE LOG POINT; count], topo || ' n=' || n || ' k=' || radix as [YFACET; Network], len as [XFACET; len], 'total' as [COLOR SHAPE; type] FROM data GROUP BY 1,3,4 UNION SELECT count_individual-count_total, count(*), topo || ' n=' || n || ' k=' || radix, len, 'loss' FROM data GROUP BY 1,3,4" --size 15x15

# ratio efficiency
results ggplot -r 'file="edge_disjoint_sets.py"' -s "100.0*count_total/count_individual as [X HIST INT LOGY; efficiency], len as [XFACET; len],  topo || ' n=' || n || ' k=' || radix as [YFACET; topo]" --size 15x15

# vertex vs. edge disjont validation
results ggplot -r '(file="edge_disjoint_paths.py" or file="vertex_disjoint_paths.py")' --datasql "SELECT count as [X; disjoint path count], count(*)*1.0/(SELECT count(*) FROM datapoints WHERE datapoints.runid = data.runid) as [Y LOG POINT LINE; pairs], len as [XFACET; len], file as [COLOR SHAPE; file], topo || ' n=' || n || ' k=' || radix as [YFACET; topo] FROM data GROUP BY 1, 3, 4, 5" --size '30x30'

# different style, including tail-disjoint paths
results ggplot2 -r '(tag="tail-disjoint-count" or file="edge_disjoint_paths.py" or file="vertex_disjoint_paths.py") and n > 500 and n < 800 and sample=1' -s "count as [X HIST INT LOGY LINE; tail-disjoint count], topo || ' n=' || n || ' k=' || radix as [YFACET; topo], file as [COLOR; type], len as [XFACET; len]" --size 15x15

# s-t map of weakly connected nodes
results ggplot2 -r 'file="edge_disjoint_paths.py"' -w '(len=4 and topo in ("DF", "JF3") or len=3 and topo in ("XP", "JF", "SF")) and count < radix*0.75 or len=1 and count > 0' -s 't*1.0/n as [Y POINT; t], s*1.0/n as [X; s], (count*100.0/radix) as [COLOR; paths (% k)], topo || "n=" || n as [FACET], "_"||len as [SHAPE; Edge vs Path]' --size 30x30

# boxplots of path counts
results ggplot2 -r 'file="edge_disjoint_paths.py" and n > 500' -s 'topo|| " n=" || n as [COLOR SHAPE; Topology], " " || len as [X; Path Length], count*100.0/radix as [Y BOX; ports (%)], (case topo in ("JF3", "DF") when 1 then "d=3" else "d=2" end) as [FACET]' --size 7x4

# small histogram plot
results ggplot2 -r 'file="vertex_disjoint_paths.py"' -w "(len=3 and radix=29 or len=4 and radix=14) and (topo != 'JF' or n=722)" -s "count as [X; # of edge disjoint paths], count(*)/2e4*100.0 as [Y POINT BAR; pairs (%)], topo || ' n=' || n as [SHAPE FILL; Network], (case topo in ('JF3', 'DF') when 1 then \"d=3 k'=14 (4 hops)\" else \"d=2 k'=29 (3 hops)\" end) as [XFACET]" -g -i s -i t --size 7x4



