import numpy as np
from math import sqrt, inf, nan
import random

import mmm_ops

from countpaths import *
from results import Results

def count_tail_disjoint_paths(graph, limit, r):
    
    g = graph
    collect_tail = r.collector(tag="tail-disjoint-count", len=Results.Int, count=Results.Int, topo=g.topo, n=g.vertices, radix=g.radix, sample=g.sample)
    collect_multi = r.collector(tag="shortest-path", len=Results.Int, pairs=Results.Int, multiplicity=Results.Text, topo=g.topo, n=g.vertices, radix=g.radix, sample=g.sample)
    collect_nonshortest = r.collector(tag="non-shortest-path", len=Results.Int, pairs=Results.Int, topo=g.topo, n=g.vertices, radix=g.radix, sample=g.sample)
    
    count = 10000
    r = np.arange(0, g.vertices, 1)
    pairs = [random.sample(list(r), 2) for _ in range(0, count)]
    
    diag = np.arange(graph.vertices)
    deg = edgetype(g.sum(0).max()) # max vertex degree
    acc = deg*np.identity(graph.vertices, dtype=edgetype)
    connected = np.zeros_like(graph, dtype=np.bool)

    for i in range(1, limit+1):
        step = mmm_ops.mmm_min(acc, graph, np.zeros_like(acc, dtype=edgetype))

            
        for s,t in pairs:
            collect_tail(len=i, count=step[s,t])
        
        step[diag, diag] = deg # full loops for other things
        
        shortest = np.multiply(step, ~connected) # elem-wise mask
        connected += (step != 0)
        
        collect_multi(multiplicity='one', pairs=(shortest == 1).sum(), len=i)
        collect_multi(multiplicity='two', pairs=(shortest == 2).sum(), len=i)
        collect_multi(multiplicity='more', pairs=(shortest > 2).sum(), len=i)
        collect_nonshortest(pairs=(step.sum()-shortest.sum()), len=i)

        acc = step
        
if __name__ == '__main__':
    r = Results()

    for topo, (files, samp) in graphnames.items():
        for i in range(1,samp+1):
            for name in files:
                if i != 1:
                    n = name[:-1] + str(i)
                else:
                    n = name
                print(n)
                g = read_graph(n)
                g.topo = topo
                g.sample = i
                count_tail_disjoint_paths(g, 5, r)
                r.commit()
    r.close()
