#!/usr/bin/env python3
from queue import PriorityQueue
from copy import deepcopy
import random
from simplepmap import pmap

class Flow:
    pass
KiB = 2**10
MiB = 2**20

# Simulate the endpoint link as a queue model with "processor" discipline (fair
# resource sharing using the distributions set above, using discrete event
# simulation.
def simulate(Lambda, linespeed, nflows, seed, collect):
    random.seed(seed)
    
    # pFabric web search upper bound
    flows = [Flow() for _ in range(21)]
    flows[1].sendBytes = 10*KiB
    flows[2].sendBytes = 10*KiB
    flows[3].sendBytes = 10*KiB
    flows[4].sendBytes = 20*KiB
    flows[5].sendBytes = 30*KiB
    flows[6].sendBytes = 30*KiB
    flows[7].sendBytes = 50*KiB
    flows[8].sendBytes = 50*KiB
    flows[9].sendBytes = 80*KiB
    flows[10].sendBytes = 80*KiB
    flows[11].sendBytes = 80*KiB
    flows[12].sendBytes = 200*KiB
    flows[13].sendBytes = 1*MiB
    flows[14].sendBytes = 1*MiB
    flows[15].sendBytes = 2*MiB
    flows[16].sendBytes = 2*MiB
    flows[17].sendBytes = 5*MiB
    flows[18].sendBytes = 5*MiB
    flows[19].sendBytes = 10*MiB
    flows[20].sendBytes = 10*MiB
    more = flows[1:] # to preserve the base-1 indexing
    for _ in range(1, nflows):
        flows += deepcopy(more)
    for i in range(1, len(flows), 2): flows[i].tOpen = random.uniform(0, nflows*10.0/Lambda)
    for i in range(2, len(flows), 2): flows[i].tOpen = random.uniform(nflows*10.0/Lambda, nflows*20.0/Lambda)

    # active is the set of flows currently running, q the queue of upcoming events.
    # We use one queue for start and finish events, which means we have to update
    # the end times frequently; it would be more efficient to keep the ends in
    # a separate queue ordered by remaining volume, but that might be harder to
    # get right.
    active = set()
    q = PriorityQueue()

    for flow in flows[1:]:
        q.put((flow.tOpen, flow))
        flow.running = False
        flow.speed = 0 # this should be multiplied by 0 later...
        flow.end = None
        flow.remaining = flow.sendBytes

    while not q.empty():
        now, f = q.get()
        if not f.running:
            # new flow, start it
            active.add(f)
            f.running = True
            f.start = now
        else:
            # some flow finished
            if now != f.end: continue # soft delete...
            active.remove(f)
            collect(size=f.sendBytes, round=flows.index(f), start=f.tOpen, time=now-f.tOpen)
            
        # in the queue, we keep one timeout for the completion of each flow.
        # when the active flows change, the throughput changes, so we have to
        # update all these timers.
        # We compute how much data was transferred at the current speed, and
        # update .start and .remaining for the next invocation.
        for f in active:
            transferred = (now-f.start)*f.speed
            f.remaining -= transferred
            assert(not f.end or f.end >= now)
            assert(f.remaining >= 0)
            f.start = now
            f.speed = linespeed/len(active)
            newend = now + f.remaining/f.speed
            f.end = newend
            q.put((f.end, f))

if __name__ == "__main__":
    ls = [300]
    from results import Results
    r = Results("etalink.db")
    def run(Lambda): 
        for linespeed in [400e6, 500e6, 600e6, 700e6, 800e6, 900e6, 1000e6, 1100e6, 1200e6]:
            for nflows in [2]:
                collect = r.collector(Lambda=Lambda, flowsize = "pfabric", linespeed = linespeed, nflows=20*nflows, size=Results.Int, round=Results.Int, start=Results.Real, time=Results.Real)
                print("lambda = %d, speed = %d, nflows = %d" % (Lambda, linespeed, 20*nflows))
                for seed in range(int(linespeed)+42, int(linespeed)+42+10000):
                    simulate(Lambda, linespeed, nflows, seed, collect)
                    r.commit()
        r.close()
    list(map(run, ls))






