from countpaths import read_graph, graphnames
from results import Results
import numpy as np
import random
from simplepmap import pmap

def index(g, s):
    return np.nonzero(np.array(g[:,s]).reshape(g.shape[0]) == 1)[0]

class Rank:
    def rank(self, st):
        r = self.r
        K = self.K
        g = self.g
        s,t = st
        return np.linalg.matrix_rank(K[index(g,s),:][:,index(g,t)])

def countpaths(g, limit, collect, count = 20000):
    
    r = np.arange(0, g.edge, 1)

    if g.edge**2 > count:
        pairs = [random.sample(list(r), 2) for _ in range(0, count)]
    else:
        pairs = [(i, j) for i in range(g.edge) for j in range(g.edge) if i != j]

    k = np.zeros(g.shape)
    k[g > 0] = np.random.sample((g > 0).sum())
    
    
    # len 1
    for s, t in pairs:
        collect(len=1, count=g[s,t], s=s, t=t)
    
    # there'd be a fast way for len2 as well, but this is a plausibilty check 
    K = np.identity(g.vertices)
    
    for i in range(2, limit+1):
        print("rank")
        rank = Rank()
        rank.r = r
        rank.K = K
        rank.g = g
        out = pmap(rank.rank, pairs)
        print("collect")
        for c, (s, t) in zip(out, pairs):
            collect(len=i, count=c, s=s, t=t)
        print("multiply")
        K = K.dot(k) + np.identity(g.vertices)
            
if __name__ == '__main__':
    r = Results()
    for topo, (files, samp) in graphnames.items():
        for i in range(1,samp+1):
            for name in files:
                if i != 1:
                    n = name[:-1] + str(i)
                else:
                    n = name
                print(n)
                g = read_graph(n)
                
                collect = r.collector(s=Results.Int, t=Results.Int, len=Results.Int, count=Results.Int, topo=topo, n=g.vertices, radix=g.radix, sample=i)
                countpaths(g, 5, collect)
                r.commit()
    r.close()

    #from ggplot import *
    #(ggplot(data, aes(x='count', fill='factor(len)')) 
        #+ geom_histogram(bins=np.arange(-0.5, 30.5, 1), alpha=0.5)
        #+ facet_grid('topo', 'len')
        ##+ ggtitle("Number of vertex-disjoint paths")
        #+ scale_y_log(base=10)
        #+ ylim(low=1, high=2e4)
    #).save("vertex_disjoint_paths.pdf", width=20, height=24)

