//
// Copyright (C) 2004 Andras Varga
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
//

#include "inet/applications/tcpapp/TCPWorkloadApp.h"

#include "inet/networklayer/common/L3AddressResolver.h"
#include "inet/common/ModuleAccess.h"

namespace inet {

Define_Module(TCPWorkloadApp);
simsignal_t TCPWorkloadApp::completeSignal = registerSignal("complete");

TCPWorkloadApp::TCPWorkloadApp() : TCPSessionApp()
{
    workloadConfigurator = nullptr;
}

void TCPWorkloadApp::initialize(int stage)
{
    TCPSessionApp::initialize(stage);
    if (stage == INITSTAGE_LOCAL) {
        cModule *node = getContainingNode(this);
        if (!node)
            throw cRuntimeError("The container @networkNode module not found");
        const char *workloadConfiguratorPath = par("workloadConfiguratorModule");
        
        if (!workloadConfiguratorPath[0])
            workloadConfigurator = nullptr;
        else {
            cModule *module = getModuleByPath(workloadConfiguratorPath);
            if (!module)
                throw cRuntimeError("Configurator module '%s' not found (check the 'workloadConfiguratorModule' parameter)", workloadConfiguratorPath);
            workloadConfigurator = check_and_cast<WorkloadConfigurator *>(module);
        }
    }
}

void TCPWorkloadApp::connect()
{
    // connect
    ASSERT(workloadConfigurator);
    std::string localNode = getContainingNode(this)->getName();
    std::string connectAddress = workloadConfigurator->getConnectAddress(localNode, par("round"));
    int connectPort = par("connectPort");

    if (connectAddress == "") return; // ignore unconfigured flows

    // we need a new connId if this is not the first connection
    socket.renewSocket();


    L3Address destination;
    L3AddressResolver().tryResolve(connectAddress.c_str(), destination);
    if (destination.isUnspecified()) {
        EV_ERROR << "Connecting to " << connectAddress << " port=" << connectPort << ": cannot resolve destination address\n";
    }
    else {
        EV_INFO << "Connecting to " << connectAddress << "(" << destination << ") port=" << connectPort << endl;
        setStatusString("connecting");

        socket.connect(destination, connectPort);

        numSessions++;
        emit(connectSignal, 1L);
    }
}

void TCPWorkloadApp::socketClosed(int connId, void *ptr)
{
    TCPSessionApp::socketClosed(connId, ptr);
    emit(completeSignal, simTime() - tOpen);
}


} // namespace inet

