//
// Copyright (C) 2012 Opensim Ltd
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
//

#include "inet/applications/tcpapp/WorkloadConfigurator.h"


namespace inet {

Define_Module(WorkloadConfigurator);

WorkloadConfigurator::WorkloadConfigurator() : cSimpleModule()
{
}

void WorkloadConfigurator::initialize(int stage)
{
    if (stage == INITSTAGE_LOCAL) {
        configuration = par("config");
        readFlows();
    }
}

void WorkloadConfigurator::readFlows()
{
    using namespace xmlutils;

    cXMLElementList flowElements = configuration->getChildrenByTagName("flow");

    for (auto & flowElement : flowElements) {
        const char *srcAttr = flowElement->getAttribute("src");
        const char *destAttr = flowElement->getAttribute("dest");
        const char *roundAttr = flowElement->getAttribute("round");
        
        try {
            
            if (PatternMatcher::containsWildcards(srcAttr) || PatternMatcher::containsWildcards(roundAttr)) {
                PatternMatcher srcMatcher(srcAttr, false, true, true);
                PatternMatcher roundMatcher(roundAttr, false, true, true);
                wildcard_match_dest.push_back(std::make_pair(
                    std::make_pair(srcMatcher, roundMatcher), destAttr));
            } else {
                exact_match_dest[std::make_pair(srcAttr, roundAttr)] = destAttr;
            }
        }
        catch (std::exception& e) {
            throw cRuntimeError("Error in XML <flow> element at %s: %s", flowElement->getSourceLocation(), e.what());
        }
    }
}

std::string  WorkloadConfigurator::getConnectAddress(std::string hostname, std::string round) {
    auto it = exact_match_dest.find(std::make_pair(hostname, round));
    if (it != exact_match_dest.end()) {
        return it->second;
    } else {
        for (auto& e : wildcard_match_dest) {
            FlowMatcher& w = e.first;
            if (w.first.matches(hostname.c_str()) && w.second.matches(round.c_str())) {
                return e.second;
            }
        }
    }
    EV << "No flow matching " << hostname << ", round " << round << "\n";
    return "";
}

} // namespace inet

