//
// Copyright (C) 2012 Opensim Ltd
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
//

#ifndef __INET_WORKLOADCONFIGURATOR_H
#define __INET_WORKLOADCONFIGURATOR_H

#include <map>
#include "inet/common/INETDefs.h"
#include "inet/common/PatternMatcher.h"
#include "inet/common/XMLUtils.h"


namespace inet {

/**
 * This module exposes the workload config to the TCPWorkloadApps performing the flows.
 */
class INET_API WorkloadConfigurator : public cSimpleModule
{
  protected:
      cXMLElement *configuration = nullptr;
      
      typedef std::pair<std::string, std::string> Flow; // host/round pair
      typedef std::pair<PatternMatcher, PatternMatcher> FlowMatcher;
      std::map<Flow, std::string> exact_match_dest;
      std::vector<std::pair<FlowMatcher, std::string>> wildcard_match_dest;
      
  public:
    /**
     * retrives connectAddress for the given host module name and round.
     */
    virtual std::string getConnectAddress(std::string hostname, std::string round);
    WorkloadConfigurator();

  protected:
    virtual int numInitStages() const override { return NUM_INIT_STAGES; }
    virtual void handleMessage(cMessage *msg) override { throw cRuntimeError("this module doesn't handle messages, it runs only in initialize()"); }
    virtual void initialize(int stage) override;
    virtual void readFlows();
};

} // namespace inet

#endif // ifndef __INET_WORKLOADCONFIGURATOR_H

