#ifndef __ECMPROUTINGTABLE_H
#define __ECMPROUTINGTABLE_H

#include <vector>
#include <omnetpp.h>
#include <array>

#include "inet/transportlayer/tcp_common/TCPSegment.h"
#include "inet/networklayer/ipv4/IPv4RoutingTable.h"

namespace inet {

class INET_API ECMPRoutingTable: public IPv4RoutingTable
{
public:
    void initialize(int stage) override;
    
    IPv4Route *findBestMatchingRoute(IPv4Datagram *datagram) const override;

    // Used to calculate which nextHop to use when multiple nextHops have equal metric.
    uint32 ecmpHash(IPv4Datagram *datagram) const;
    
private:
    // A cache structured as a ring buffer of LRU routes.
    // we store vectors since obviously we need to select depending on ECMP.
    // cacheptr always points to the slot to be overwritten next and oves forwards.
    mutable size_t cacheptr = 0;
    mutable std::array<std::pair<IPv4Address, RouteVector>, 256> routeCache;
    
    uint32 routerid;
};

Define_Module(ECMPRoutingTable);

}

#endif

