#include "LetFlowRoutingTable.h"

namespace inet {
using namespace tcp;

Define_Module(LetFlowRoutingTable);

simsignal_t LetFlowRoutingTable::rerouteSignal = registerSignal("reroute");

std::ostream& operator<<(std::ostream& os, const LetFlowRoutingTable::FourTuple& tup)
{
    os << "(" << std::get<0>(tup) << ", " << std::get<1>(tup) << ", " << std::get<2>(tup) << ", " << std::get<3>(tup) << ")";
    return os;
}

IPv4Route *LetFlowRoutingTable::findBestMatchingRoute(IPv4Datagram *datagram) const
{
    Enter_Method("findBestMatchingRoute(IPv4Datagram *)");
    
    //  extract 4-tuple.
    IPv4Address srcAddr = datagram->getSrcAddress();
    IPv4Address dstAddr = datagram->getDestAddress();
    uint16_t srcPort = 0;
    uint16_t dstPort = 0;
    if (datagram->getTransportProtocol() == IP_PROT_TCP) {
        TCPSegment* tcpPacket = (TCPSegment*) datagram->getEncapsulatedPacket();
        srcPort = tcpPacket->getSrcPort();
        dstPort = tcpPacket->getDestPort();
    }
    FourTuple tup = std::make_tuple(srcAddr, dstAddr, srcPort, dstPort);
    
    // check flow table, and check age
    auto it = flowtable.find(tup);
    if (it != flowtable.end()) {
        auto flowroute = it->second;
        if ((simTime() - std::get<1>(flowroute)) < flowletgap) {
            // route is valid, use it
            EV_DEBUG << "flowtable: valid route " << std::get<0>(flowroute)->info() << " for " << tup << "\n";
            return std::get<0>(flowroute);
        } else {
            EV_DEBUG << "flowtable: stale route t=" << std::get<1>(flowroute) << " for " << tup << "\n";
            flowtable.erase(it);
        }
    }
    
    // there was no flow entry, or the flowlet timed out. Reroute,
    auto routes = findBestMatchingRoutes(datagram->getDestAddress());
    if (routes.empty()) return nullptr;
    
    int choice = intrand(routes.size());
    auto flowroute = std::make_tuple(routes[choice], simTime());
    
    flowtable[tup] = flowroute;

    // emit is not const. Actually the problem is more that findBestMatchingRoute is const,
    // but both is hard to change.
    auto me = const_cast<LetFlowRoutingTable*>(this);
    me->emit(rerouteSignal, routes.size());

    EV << "flowtable: new route " << std::get<0>(flowroute)->info() << " for " << tup << " choice " << choice << " of " << routes.size() << "\n";
    return std::get<0>(flowroute);
}

void LetFlowRoutingTable::initialize(int stage) {
    IPv4RoutingTable::initialize(stage);
    if (stage == INITSTAGE_LOCAL) {
        flowletgap = par("flowletgap");
    }
}

}
