#include "MPTCPVirtualDataSendQueue.h"

namespace inet {

Register_Class(MPTCPVirtualDataSendQueue);

MPTCPVirtualDataSendQueue::MPTCPVirtualDataSendQueue(MPTCPConnection *_conn)
{
    begin = end = 0;
    conn = _conn;
}

MPTCPVirtualDataSendQueue::~MPTCPVirtualDataSendQueue()
{
}

void MPTCPVirtualDataSendQueue::init(int subflowId, uint32 startSeq)
{
    subflowBeginMap[subflowId] = startSeq;
    subflowEndMap[subflowId] = startSeq;
}

std::string MPTCPVirtualDataSendQueue::info(int subflowId)
{
    std::stringstream out;
    out << " [" << subflowBeginMap[subflowId] << ".." << subflowEndMap[subflowId] << ".." << end << ")";
    return out.str();
}

void MPTCPVirtualDataSendQueue::enqueueAppData(cPacket *msg)
{
    EV << " enqueueAppData(bytes=" << msg->getByteLength() << ")\n";
    for (int subflowId = 0; subflowId < conn->subflows.size(); subflowId++)
        EV << "sendQ(subflow id=" << subflowId << "): " << info(subflowId) << "\n";
    end += msg->getByteLength();
    //if (seqLess(subflowEnd[0], subflowBegin[0]))
    //    throw cRuntimeError("Send queue is full");
    delete msg;
}

uint32 MPTCPVirtualDataSendQueue::getBufferStartSeq(int subflowId)
{
    return subflowBeginMap[subflowId];
}

uint32 MPTCPVirtualDataSendQueue::getBufferEndSeq(int subflowId)
{
    return subflowEndMap[subflowId] + end;
}

TCPSegment *MPTCPVirtualDataSendQueue::createSegmentWithBytes(int subflowId, uint32 fromSeq, ulong numBytes)
{
    EV << "sendQ: " << info(subflowId) << " createSeg(seq=" << fromSeq << " len=" << numBytes << ")\n";

    ASSERT(seqLE(getBufferStartSeq(subflowId), fromSeq) && seqLE(fromSeq + numBytes, getBufferEndSeq(subflowId)));

    if (end >= numBytes) {
        end -= numBytes;
        subflowEndMap[subflowId] += numBytes;
    }

    char msgname[50];
    sprintf(msgname, "mptcpseg(id=%d, l=%lu)", subflowId, numBytes);

    TCPSegment *tcpseg = new TCPSegment(msgname);
    tcpseg->setSequenceNo(fromSeq);
    tcpseg->setPayloadLength(numBytes);
    return tcpseg;
}

void MPTCPVirtualDataSendQueue::discardUpTo(int subflowId, uint32 seqNum)
{
    EV << "sendQ: " << info(subflowId) << " discardUpTo(seq=" << seqNum << ")\n";

    ASSERT(seqLE(subflowBeginMap[subflowId], seqNum) && seqLE(seqNum, subflowEndMap[subflowId]));

    subflowBeginMap[subflowId] = seqNum;
}


void MPTCPVirtualDataSendQueue::reserveUpTo(int subflowId, uint32 seqNum)
{
    EV << "sendQ: " << info(subflowId) << " reserveUpTo(subflowId=" << subflowId << ", seq=" << seqNum << ")\n";

    end -= seqNum - subflowEndMap[subflowId];
    subflowEndMap[subflowId] = seqNum;
}


}
