//
// Copyright (C) 2013 Opensim Ltd.
// Author: Levente Meszaros
// Author: Marcel Schneider (TCP Version)
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
//

#include "inet/transportlayer/tcp/TCPMultiplexer.h"
#include "inet/transportlayer/tcp/TCP.h"

namespace inet {
namespace tcp {

Define_Module(TCPMultiplexer);

void TCPMultiplexer::handleMessage(cMessage *message)
{
    cGate *arrivalGate = message->getArrivalGate();
    const char *arrivalGateName = arrivalGate->getBaseName();
    int gateIndex = arrivalGate->getIndex();
    if (!strcmp(arrivalGateName, "tcpIn")) {
        send(message, "appOut", connIdToGate[getConnId(message)]);
    } else if (!strcmp(arrivalGateName, "appIn")) {
        connIdToGate[getConnId(message)] = gateIndex;
        send(message, "tcpOut");
    } else {
        throw cRuntimeError("Unknown arrival gate");
    }
}

int TCPMultiplexer::getConnId(cMessage *msg)
{
    TCPCommand *controlInfo = check_and_cast<TCPCommand *>(msg->getControlInfo());
    int connId = controlInfo->getConnId();
    return connId;
}

} // namespace tcp
} // namespace inet

