//
// Copyright (C) 2004-2005 Andras Varga
// Copyright (C) 2009 Thomas Reschka
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
//

#include "inet/transportlayer/tcp/flavours/FlowletDCTCP.h"

namespace inet {
using namespace tcp;

Register_Class(FlowletDCTCP);

FlowletDCTCP::FlowletDCTCP() : DCTCP() {
}

void FlowletDCTCP::switchLayer()
{
    if (conn->remoteAddr.getType() == L3Address::IPv4) {
        uint32 choice = conn->tcpMain->intrand(0xFFFFFFFF);
        uint32 layerchange = choice & conn->tcpMain->layerMask;
        uint32 dstbits = conn->remoteAddr.toIPv4().getInt();
        conn->remoteAddr = IPv4Address(dstbits ^ layerchange);
        if (conn->localAddr.getType() == L3Address::IPv4) {
            uint32 srcbits = conn->localAddr.toIPv4().getInt();
            conn->localAddr  = IPv4Address(srcbits ^ layerchange);
        }
    } else {
        EV_WARN << "Layered switching only implemented for IPv4.\n";
    }
}
    

void FlowletDCTCP::receivedDataAck(uint32 firstSeqAcked)
{
    uint32 old_cwnd = state->snd_cwnd;
    DCTCP::receivedDataAck(firstSeqAcked);
    uint32 end_cwnd = state->snd_cwnd;
    if (end_cwnd < old_cwnd) {
        // the cwnd decreased, so we will produce a gap.
        // We could check for some min size here, but most of the time it will
        // be large -- except if we have a very congested link. But then it
        // is probably worth the reordering to just switch quickly.
        switchLayer();
        EV_INFO << "FlowletDCTCP: cwnd update " << conn->connId << ": " 
            << old_cwnd << " " << end_cwnd << ", switching layer to " 
            << conn->localAddr << "->" << conn->remoteAddr << "\n";
    }
}

void FlowletDCTCP::processRexmitTimer(TCPEventCode& event)
{
    switchLayer();
    EV_INFO << "FlowletDCTCP: rto rexmit " << conn->connId << ": " 
        "switching layer to " << conn->localAddr << "->" << conn->remoteAddr << "\n";
    DCTCP::processRexmitTimer(event);
}

void FlowletDCTCP::packetLost()
{
    if (rexmitTimer->isScheduled()) {
        cancelEvent(rexmitTimer);
        conn->scheduleTimeout(rexmitTimer, 0);
        EV_INFO << "FlowletDCTCP: packet lost, scheduling rexmit now\n";
    } else {
        switchLayer();
        EV_INFO << "FlowletDCTCP: packet lost with no rexmit scheduled, "
            "switching layer to " << conn->localAddr << "->" << conn->remoteAddr << "\n";
    }
}

}
