#!/usr/bin/env python3
import random

def output_graph(graph, file=None):
    print(len(graph), int(sum(len(n) for n in graph) / 2), file=file)
    for node in graph:
       print( " ".join(str(e+1) for e in node) + " ", file=file)
       
def make_permutation(n):
    l = list(map(lambda i: [i], range(0, n)))
    random.shuffle(l)
    return l

def map_blocked(graph, size):
    n = len(graph)
    g = []
    for n in graph:
        for i in range(0, size):
            out = []
            for t in n:
                out += list(range(t*size, (t+1)*size))
            g.append(out)
    return g

def multiply(graph, factor):
    return [n * factor for n in graph]

if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser(prog='make_workload.py')
    parser.add_argument('-n', help='Number of routers.', type=int, required=True)
    parser.add_argument('-p', help='Number of endpoints per router.', type=int, default=1)
    parser.add_argument('--fraction', help='Fraction of endpoints participating in communication.', type=float, default=1.0)
    parser.add_argument('--blocksize', help='Blocksize for blocked random mapping.', type=int, default=None)
    parser.add_argument('--seed', help='Random seed', type=int, default=42)
    parser.add_argument('--multiply', help='Use this number of flows per connection.', type=int, default=1)
    
    args = parser.parse_args()
    
    random.seed(args.seed)
    graph = make_permutation(int(args.n*args.p/args.blocksize))
    mapped = map_blocked(graph, args.blocksize)
    multi = multiply(mapped, args.multiply)
    
    with open("groupexchange_%d_%d.txt" % (args.n, args.p), "w") as f:
        output_graph(multi, f)
        print(f.name) # only stdout output, read by driver
